@@ -51,10 +51,11 @@
 #include "const-info.h"
 #include "gldns/gbuffer.h"
 #include "gldns/wire2str.h"
+#include "gldns/parseutil.h"
 
 
 static char *_json_ptr_first(const struct mem_funcs *mf,
-    const char *jptr, char *first, size_t first_sz)
+    const char *jptr, char *first, ssize_t first_sz)
 {
 	const char *next_ref, *k;
 	char *j;
@@ -65,7 +66,7 @@ static char *_json_ptr_first(const struct mem_funcs *mf,
 	if (!(next_ref = strchr(jptr, '/')))
 		next_ref = strchr(jptr, '\0');
 
-	if (next_ref - jptr > first_sz)
+	if ((ssize_t)(next_ref - jptr + 1) > first_sz || !first)
 		first = GETDNS_XMALLOC(*mf, char, next_ref - jptr + 1);
 
 	for (j = first, k = jptr; k < next_ref; j++, k++)
@@ -280,8 +281,7 @@ getdns_dict_get_names(const getdns_dict *dict, getdns_list **answer)
 
 	RBTREE_FOR(item, struct getdns_dict_item *,
 		(_getdns_rbtree_t *)&(dict->root)) {
-		_getdns_list_append_const_bindata(*answer,
-		    strlen(item->node.key) + 1, item->node.key);
+		_getdns_list_append_string(*answer, item->node.key);
 	}
 	return GETDNS_RETURN_GOOD;
 }				/* getdns_dict_get_names */
@@ -484,7 +484,6 @@ _getdns_dict_copy(const struct getdns_dict * srcdict,
 	if (!*dstdict)
 		return GETDNS_RETURN_GENERIC_ERROR;
 
-	retval = GETDNS_RETURN_GOOD;
 	RBTREE_FOR(item, struct getdns_dict_item *,
 		(struct _getdns_rbtree_t *)&(srcdict->root)) {
 		key = (char *) item->node.key;
@@ -508,6 +507,9 @@ _getdns_dict_copy(const struct getdns_dict * srcdict,
 			retval = getdns_dict_set_list(*dstdict, key,
 				item->i.data.list);
 			break;
+		default:
+			retval = GETDNS_RETURN_WRONG_TYPE_REQUESTED;
+			break;
 		}
 		if (retval != GETDNS_RETURN_GOOD) {
 			getdns_dict_destroy(*dstdict);;
@@ -530,53 +532,95 @@ getdns_dict_destroy(struct getdns_dict *dict)
 
 /*---------------------------------------- getdns_dict_set_dict */
 getdns_return_t
-getdns_dict_set_dict(
-    getdns_dict *dict, const char *name, const getdns_dict *child_dict)
+_getdns_dict_set_this_dict(
+    getdns_dict *dict, const char *name, getdns_dict *child_dict)
 {
 	getdns_item    *item;
-	getdns_dict    *newdict;
 	getdns_return_t r;
 
 	if (!dict || !name || !child_dict)
 		return GETDNS_RETURN_INVALID_PARAMETER;
 
-	if ((r = _getdns_dict_copy(child_dict, &newdict)))
+	if ((r = _getdns_dict_find_and_add(dict, name, &item)))
 		return r;
 
-	if ((r = _getdns_dict_find_and_add(dict, name, &item))) {
-		getdns_dict_destroy(newdict);
-		return r;
-	}
 	item->dtype = t_dict;
-	item->data.dict = newdict;
+	item->data.dict = child_dict;
 	return GETDNS_RETURN_GOOD;
 }				/* getdns_dict_set_dict */
 
+getdns_return_t
+getdns_dict_set_dict(
+    getdns_dict *dict, const char *name, const getdns_dict *child_dict)
+{
+	getdns_dict    *newdict;
+	getdns_return_t r;
+
+	if ((r = _getdns_dict_copy(child_dict, &newdict)))
+		return r;
+
+	if ((r = _getdns_dict_set_this_dict(dict, name, newdict)))
+		getdns_dict_destroy(newdict);
+
+	return r;
+}
+
+
 /*---------------------------------------- getdns_dict_set_list */
 getdns_return_t
-getdns_dict_set_list(
-    getdns_dict *dict, const char *name, const getdns_list *child_list)
+_getdns_dict_set_this_list(
+    getdns_dict *dict, const char *name, getdns_list *child_list)
 {
 	getdns_item    *item;
-	getdns_list    *newlist;
 	getdns_return_t r;
 
 	if (!dict || !name || !child_list)
 		return GETDNS_RETURN_INVALID_PARAMETER;
 
-	if ((r = _getdns_list_copy(child_list, &newlist)))
+	if ((r = _getdns_dict_find_and_add(dict, name, &item)))
 		return r;
 
-	if ((r = _getdns_dict_find_and_add(dict, name, &item))) {
-		getdns_list_destroy(newlist);
-		return r;
-	}
 	item->dtype = t_list;
-	item->data.list = newlist;
+	item->data.list = child_list;
 	return GETDNS_RETURN_GOOD;
 }				/* getdns_dict_set_list */
 
+getdns_return_t
+getdns_dict_set_list(
+    getdns_dict *dict, const char *name, const getdns_list *child_list)
+{
+	getdns_list    *newlist;
+	getdns_return_t r;
+
+	if ((r = _getdns_list_copy(child_list, &newlist)))
+		return r;
+
+	if ((r = _getdns_dict_set_this_list(dict, name, newlist)))
+		getdns_list_destroy(newlist);
+
+	return r;
+}
+
+
 /*---------------------------------------- getdns_dict_set_bindata */
+getdns_return_t
+_getdns_dict_set_this_bindata(
+    getdns_dict *dict, const char *name, getdns_bindata *bindata)
+{
+	getdns_item    *item;
+	getdns_return_t r;
+
+	if (!dict || !name || !bindata)
+		return GETDNS_RETURN_INVALID_PARAMETER;
+
+	if ((r = _getdns_dict_find_and_add(dict, name, &item)))
+		return r;
+
+	item->dtype = t_bindata;
+	item->data.bindata = bindata;
+	return GETDNS_RETURN_GOOD;
+}
+
 getdns_return_t
 _getdns_dict_set_const_bindata(
     getdns_dict *dict, const char *name, size_t size, const void *data)
@@ -598,7 +642,7 @@ _getdns_dict_set_const_bindata(
 	item->dtype = t_bindata;
 	item->data.bindata = newbindata;
 	return GETDNS_RETURN_GOOD;
-}				/* getdns_dict_set_bindata */
+}
 
 getdns_return_t
 getdns_dict_set_bindata(
@@ -613,9 +657,26 @@ getdns_dict_set_bindata(
 getdns_return_t
 getdns_dict_util_set_string(getdns_dict *dict, char *name, const char *value)
 {
-	return value
-	    ? _getdns_dict_set_const_bindata(dict, name, strlen(value), value)
-	    : GETDNS_RETURN_INVALID_PARAMETER;
+	getdns_item    *item;
+	getdns_bindata *newbindata;
+	getdns_return_t r;
+
+	if (!dict || !name || !value)
+		return GETDNS_RETURN_INVALID_PARAMETER;
+
+	if (!(newbindata = _getdns_bindata_copy(
+	    &dict->mf, strlen(value) + 1, (uint8_t *)value)))
+		return GETDNS_RETURN_MEMORY_ERROR;
+
+	newbindata->size -= 1;
+
+	if ((r = _getdns_dict_find_and_add(dict, name, &item))) {
+		_getdns_bindata_destroy(&dict->mf, newbindata);
+		return r;
+	}
+	item->dtype = t_bindata;
+	item->data.bindata = newbindata;
+	return GETDNS_RETURN_GOOD;
 }				/* getdns_dict_util_set_dict */
 
 /*---------------------------------------- getdns_dict_set_int */
@@ -671,18 +732,44 @@ _getdns_bindata_is_dname(getdns_bindata *bindata)
 		bindata->data[bindata->size - 1] == 0;
 }
 
+static int
+getdns_pp_base64(gldns_buffer *buf, getdns_bindata *bindata)
+{
+	size_t p = gldns_buffer_position(buf);
+	size_t base64str_sz;
+	char *target;
+	size_t avail;
+
+	if (gldns_buffer_printf(buf, " <bindata of ") < 0)
+		return -1;
+
+	base64str_sz = gldns_b64_ntop_calculate_size(bindata->size);
+	target = (char *)gldns_buffer_current(buf);
+	avail = gldns_buffer_remaining(buf);
+	if (avail >= base64str_sz)
+		gldns_buffer_skip(buf, gldns_b64_ntop(
+		    bindata->data, bindata->size,
+		    target, base64str_sz));
+	else
+		gldns_buffer_skip(buf, base64str_sz);
+
+	if (gldns_buffer_printf(buf, ">") < 0)
+		return -1;
+
+	return gldns_buffer_position(buf) - p;
+}
+
 /*---------------------------------------- getdns_pp_bindata */
 /**
  * private function to pretty print bindata to a gldns_buffer
  * @param buf     buffer to write to
- * @param indent  number of spaces to append after newline
  * @param bindata the bindata to print
  * @return        on success the number of written characters
  *                if an output error is encountered, a negative value
  */
 static int
-getdns_pp_bindata(gldns_buffer *buf, size_t indent,
-	getdns_bindata *bindata, int rdata_raw, int json)
+getdns_pp_bindata(gldns_buffer *buf, getdns_bindata *bindata,
+    int rdata_raw, int json)
 {
 	size_t i, p = gldns_buffer_position(buf);
 	uint8_t *dptr;
@@ -722,7 +809,7 @@ getdns_pp_bindata(gldns_buffer *buf, size_t indent,
 		(void)gldns_wire2str_dname_buf(
 		    bindata->data, bindata->size, spc, sizeof(spc));
 		if (gldns_buffer_printf(
-		    buf, (json ? "\"%s\"" : "of \"%s\">"), spc) < 0)
+		    buf, (json ? "\"%s\"" : "for %s>"), spc) < 0)
 			return -1;
 	} else if (json) {
 		if (gldns_buffer_printf(buf, "[") < 0)
@@ -827,7 +914,7 @@ getdns_pp_list(gldns_buffer *buf, size_t indent, const getdns_list *list,
 			    GETDNS_RETURN_GOOD)
 				return -1;
 			if (getdns_pp_bindata(
-			    buf, indent, bindata_item, 0, json) < 0)
+			    buf, bindata_item, 0, json) < 0)
 				return -1;
 			break;
 
@@ -993,7 +1080,7 @@ getdns_pp_dict(gldns_buffer * buf, size_t indent,
 			     strcmp(item->node.key, "status") == 0 ||
 			     strcmp(item->node.key, "append_name") == 0 ||
 			     strcmp(item->node.key, "follow_redirects") == 0 ||
-				 strcmp(item->node.key, "transport") == 0 ||
+			     strcmp(item->node.key, "transport") == 0 ||
 			     strcmp(item->node.key, "resolution_type") == 0 ||
 			     strcmp(item->node.key, "tls_authentication") == 0 ) &&
 			    (strval =
@@ -1035,8 +1122,18 @@ getdns_pp_dict(gldns_buffer * buf, size_t indent,
 				              )) < 0)
 					return -1;
 	
+			} else if (!json &&
+			    (strcmp(item->node.key, "pin-sha256") == 0 ||
+			     strcmp(item->node.key, "value") == 0) &&
+			     item->i.data.bindata->size > 0 &&
+			     item->i.data.bindata->size % 4 == 0) {
+
+				if (getdns_pp_base64(buf,
+				    item->i.data.bindata) < 0)
+					return -1;
+
 			} else if (getdns_pp_bindata(
-			    buf, indent, item->i.data.bindata,
+			    buf, item->i.data.bindata,
 			    (strcmp(item->node.key, "rdata_raw") == 0),
 			    json) < 0)
 				return -1;
@@ -1127,9 +1224,9 @@ getdns_pretty_snprint_dict(char *str, size_t size, const getdns_dict *dict)
 
 	if (!dict) return -1;
 
-	gldns_buffer_init_frm_data(&buf, str, size);
+	gldns_buffer_init_vfixed_frm_data(&buf, str, size);
 	return getdns_pp_dict(&buf, 0, dict, 0) < 0
-	     ? -1 : gldns_buffer_position(&buf);
+	     ? -1 : (int)gldns_buffer_position(&buf);
 }
 
 char *
@@ -1161,9 +1258,9 @@ getdns_pretty_snprint_list(char *str, size_t size, const getdns_list *list)
 
 	if (!list) return -1;
 
-	gldns_buffer_init_frm_data(&buf, str, size);
+	gldns_buffer_init_vfixed_frm_data(&buf, str, size);
 	return getdns_pp_list(&buf, 0, list, 0, 0) < 0
-	     ? -1 : gldns_buffer_position(&buf);
+	     ? -1 : (int)gldns_buffer_position(&buf);
 }
 
 char *
@@ -1196,9 +1293,9 @@ getdns_snprint_json_dict(
 
 	if (!dict) return -1;
 
-	gldns_buffer_init_frm_data(&buf, str, size);
+	gldns_buffer_init_vfixed_frm_data(&buf, str, size);
 	return getdns_pp_dict(&buf, 0, dict, pretty ? 1 : 2) < 0
-	     ? -1 : gldns_buffer_position(&buf);
+	     ? -1 : (int)gldns_buffer_position(&buf);
 }
 
 char *
@@ -1231,9 +1328,9 @@ getdns_snprint_json_list(
 
 	if (!list) return -1;
 
-	gldns_buffer_init_frm_data(&buf, str, size);
+	gldns_buffer_init_vfixed_frm_data(&buf, str, size);
 	return getdns_pp_list(&buf, 0, list, 0, pretty ? 1 : 2) < 0
-	     ? -1 : gldns_buffer_position(&buf);
+	     ? -1 : (int)gldns_buffer_position(&buf);
 }
 
 /* dict.c */
