@@ -38,114 +38,52 @@
 #include <stdio.h>
 #include <string.h>
 #include "config.h"
+#include "general.h"
+#ifdef HAVE_LIBUNBOUND
+#include <unbound.h>
+#endif
+#ifdef HAVE_UNBOUND_EVENT_API
+#include "ub_loop.h"
+#endif
 #include "gldns/wire2str.h"
 #include "context.h"
 #include "types-internal.h"
 #include "util-internal.h"
 #include "dnssec.h"
 #include "stub.h"
-#include "general.h"
+#include "dict.h"
+#include "mdns.h"
+#include "debug.h"
 
-/* cancel, cleanup and send timeout to callback */
-static void
-ub_resolve_timeout(void *arg)
+void _getdns_call_user_callback(getdns_dns_req *dnsreq, getdns_dict *response)
 {
-	getdns_dns_req *dns_req = (getdns_dns_req *) arg;
-	(void) _getdns_context_request_timed_out(dns_req);
-}
+	_getdns_context_clear_outbound_request(dnsreq);
 
-void _getdns_call_user_callback(getdns_dns_req *dns_req,
-    struct getdns_dict *response)
-{
-	struct getdns_context *context = dns_req->context;
-	getdns_transaction_t trans_id = dns_req->trans_id;
-	getdns_callback_t cb = dns_req->user_callback;
-	void *user_arg = dns_req->user_pointer;
-
-	/* clean up */
-	_getdns_context_clear_outbound_request(dns_req);
-	_getdns_dns_req_free(dns_req);
-
-	context->processing = 1;
-	cb(context,
-	    (response ? GETDNS_CALLBACK_COMPLETE : GETDNS_CALLBACK_ERROR),
-	    response, user_arg, trans_id);
-	context->processing = 0;
+	if (dnsreq->user_callback) {
+		dnsreq->context->processing = 1;
+		dnsreq->user_callback(dnsreq->context,
+		    ( ! response                ? GETDNS_CALLBACK_ERROR
+		    : dnsreq->request_timed_out ? GETDNS_CALLBACK_TIMEOUT
+		                                : GETDNS_CALLBACK_COMPLETE ),
+		    response, dnsreq->user_pointer, dnsreq->trans_id);
+		dnsreq->context->processing = 0;
+	}
+	_getdns_dns_req_free(dnsreq);
 }
 
 static int
 no_answer(getdns_dns_req *dns_req)
 {
 	getdns_network_req **netreq_p, *netreq;
-	int new_canonical = 0;
-	uint8_t canon_spc[256];
-	const uint8_t *canon;
-	size_t canon_len;
-	uint8_t owner_spc[256];
-	const uint8_t *owner;
-	size_t owner_len;
-
-	_getdns_rr_iter rr_spc, *rr;
-	_getdns_rdf_iter rdf_spc, *rdf;
 
 	for (netreq_p = dns_req->netreqs; (netreq = *netreq_p); netreq_p++) {
-		if (netreq->response_len == 0 ||
-		    GLDNS_ANCOUNT(netreq->response) == 0)
-			continue;
-		canon = netreq->owner->name;
-		canon_len = netreq->owner->name_len;
-		if (netreq->request_type != GETDNS_RRTYPE_CNAME
-		    && GLDNS_ANCOUNT(netreq->response) > 1) do {
-			new_canonical = 0;
-			for ( rr = _getdns_rr_iter_init(&rr_spc
-			                               , netreq->response
-			                               , netreq->response_len)
-			    ; rr && _getdns_rr_iter_section(rr)
-			         <= GLDNS_SECTION_ANSWER
-			    ; rr = _getdns_rr_iter_next(rr)) {
+		_getdns_rrset_spc answer;
 
-				if (_getdns_rr_iter_section(rr) !=
-				    GLDNS_SECTION_ANSWER)
-					continue;
-
-				if (gldns_read_uint16(rr->rr_type) !=
-				    GETDNS_RRTYPE_CNAME)
-					continue;
-				
-				owner = _getdns_owner_if_or_as_decompressed(
-				    rr, owner_spc, &owner_len);
-				if (!_getdns_dname_equal(canon, owner))
-					continue;
-
-				if (!(rdf = _getdns_rdf_iter_init(
-				    &rdf_spc, rr)))
-					continue;
-
-				canon = _getdns_rdf_if_or_as_decompressed(
-				    rdf, canon_spc, &canon_len);
-				new_canonical = 1;
-			}
-		} while (new_canonical);
-		for ( rr = _getdns_rr_iter_init(&rr_spc
-					       , netreq->response
-					       , netreq->response_len)
-		    ; rr && _getdns_rr_iter_section(rr)
-			 <= GLDNS_SECTION_ANSWER
-		    ; rr = _getdns_rr_iter_next(rr)) {
-
-			if (_getdns_rr_iter_section(rr) !=
-			    GLDNS_SECTION_ANSWER)
-				continue;
-
-			if (gldns_read_uint16(rr->rr_type) !=
-			    netreq->request_type)
-				continue;
-			
-			owner = _getdns_owner_if_or_as_decompressed(
-			    rr, owner_spc, &owner_len);
-			if (_getdns_dname_equal(canon, owner))
-				return 0;
-		}
+		if (netreq->response_len > 0 &&
+		    GLDNS_ANCOUNT(netreq->response) > 0 &&
+		    _getdns_rrset_answer(&answer, netreq->response
+		                                , netreq->response_len))
+			return 0;
 	}
 	return 1;
 }
@@ -154,18 +92,28 @@ void
 _getdns_check_dns_req_complete(getdns_dns_req *dns_req)
 {
 	getdns_network_req **netreq_p, *netreq;
-	int results_found = 0;
+	int results_found = 0, timed_out = 1, r;
+	uint64_t now_ms  = 0;
 	
 	for (netreq_p = dns_req->netreqs; (netreq = *netreq_p); netreq_p++)
-		if (netreq->state != NET_REQ_FINISHED &&
-		    netreq->state != NET_REQ_CANCELED)
+		if (!_getdns_netreq_finished(netreq))
 			return;
-		else if (netreq->response_len > 0)
-			results_found = 1;
+		else {
+			if (netreq->state != NET_REQ_TIMED_OUT)
+				timed_out = 0;
+			if (netreq->response_len > 0)
+				results_found = 1;
+		}
+
+	if (timed_out) {
+		_getdns_context_request_timed_out(dns_req);
+		return;
+	}
 
 	/* Do we have to check more suffixes on nxdomain/nodata?
 	 */
-	if (dns_req->suffix_appended && /* Something was appended */
+	if (dns_req->is_dns_request &&
+	    dns_req->suffix_appended && /* Something was appended */
 	    dns_req->suffix_len > 1 &&  /* Next suffix available */
 	    no_answer(dns_req)) {
 		/* Remove suffix from name */
@@ -184,18 +132,24 @@ _getdns_check_dns_req_complete(getdns_dns_req *dns_req)
 			}
 		} while (dns_req->suffix_len > 1 && *dns_req->suffix);
 		if (dns_req->append_name == GETDNS_APPEND_NAME_ALWAYS ||
+		    dns_req->append_name ==
+		    GETDNS_APPEND_NAME_TO_SINGLE_LABEL_FIRST ||
 		    (dns_req->suffix_len > 1 && *dns_req->suffix)) {
 			for ( netreq_p = dns_req->netreqs
 			    ; (netreq = *netreq_p)
 			    ; netreq_p++ ) {
 				_getdns_netreq_reinit(netreq);
-				if (_getdns_submit_netreq(netreq))
-					netreq->state = NET_REQ_FINISHED;
+				if ((r = _getdns_submit_netreq(netreq, &now_ms))) {
+					if (r == DNS_REQ_FINISHED)
+						return;
+					_getdns_netreq_change_state(netreq, NET_REQ_ERRORED);
+				}
 			}
 			_getdns_check_dns_req_complete(dns_req);
 			return;
 		}
 	} else if (
+	    dns_req->is_dns_request &&
 	    ( dns_req->append_name ==
 	      GETDNS_APPEND_NAME_ONLY_TO_SINGLE_LABEL_AFTER_FAILURE ||
 	      dns_req->append_name ==
@@ -224,18 +178,32 @@ _getdns_check_dns_req_complete(getdns_dns_req *dns_req)
 			    ; (netreq = *netreq_p)
 			    ; netreq_p++ ) {
 				_getdns_netreq_reinit(netreq);
-				if (_getdns_submit_netreq(netreq))
-					netreq->state = NET_REQ_FINISHED;
+				if ((r = _getdns_submit_netreq(netreq, &now_ms))) {
+					if (r == DNS_REQ_FINISHED)
+						return;
+					_getdns_netreq_change_state(netreq, NET_REQ_ERRORED);
+				}
 			}
 			_getdns_check_dns_req_complete(dns_req);
 			return;
 		}
 	}
+#if defined(REQ_DEBUG) && REQ_DEBUG
 	if (dns_req->internal_cb)
+		debug_req("CB Internal", *dns_req->netreqs);
+	else if (results_found)
+		debug_req("CB Complete", *dns_req->netreqs);
+	else
+		debug_req("CB Error   ", *dns_req->netreqs);
+#endif
+	if (dns_req->internal_cb) {
+		_getdns_context_clear_outbound_request(dns_req);
 		dns_req->internal_cb(dns_req);
-	else if (! results_found)
+	} else if (! results_found)
 		_getdns_call_user_callback(dns_req, NULL);
-	else if (dns_req->dnssec_return_validation_chain
+	else if (
+	    dns_req->is_dns_request &&
+	    (dns_req->dnssec_return_validation_chain
 #ifdef DNSSEC_ROADBLOCK_AVOIDANCE
 	    || (   dns_req->dnssec_roadblock_avoidance 
 	       && !dns_req->avoid_dnssec_roadblocks)
@@ -244,30 +212,64 @@ _getdns_check_dns_req_complete(getdns_dns_req *dns_req)
 #ifdef STUB_NATIVE_DNSSEC
 	    || (dns_req->context->resolution_type == GETDNS_RESOLUTION_STUB
 	        && (dns_req->dnssec_return_status ||
-	            dns_req->dnssec_return_only_secure
+	            dns_req->dnssec_return_only_secure ||
+	            dns_req->dnssec_return_all_statuses
 	           ))
 #endif
-	    )
+	    )) {
+		/* Reschedule timeout for this DNS request
+		 */
+		if (dns_req->timeout.timeout_cb && dns_req->timeout.ev)
+			GETDNS_CLEAR_EVENT(dns_req->loop, &dns_req->timeout);
+
+		GETDNS_SCHEDULE_EVENT(dns_req->loop, -1,
+		    _getdns_ms_until_expiry2(dns_req->expires, &now_ms),
+		    getdns_eventloop_event_init(&dns_req->timeout, dns_req,
+		    NULL, NULL, (getdns_eventloop_callback)
+		    _getdns_validation_chain_timeout));
+
 		_getdns_get_validation_chain(dns_req);
-	else
+	} else
 		_getdns_call_user_callback(
 		    dns_req, _getdns_create_getdns_response(dns_req));
 }
 
 #ifdef HAVE_LIBUNBOUND
+#ifdef HAVE_UNBOUND_EVENT_API
+static void
+ub_resolve_event_callback(void* arg, int rcode, void *pkt, int pkt_len,
+    int sec, char* why_bogus)
+{
+	getdns_network_req *netreq = (getdns_network_req *) arg;
+	getdns_dns_req *dns_req = netreq->owner;
+
+	_getdns_netreq_change_state(netreq, NET_REQ_FINISHED);
+	/* parse */
+	if (getdns_apply_network_result(
+	    netreq, rcode, pkt, pkt_len, sec, why_bogus)) {
+		_getdns_call_user_callback(dns_req, NULL);
+		return;
+	}
+	_getdns_check_dns_req_complete(dns_req);
+
+} /* ub_resolve_event_callback */
+#endif
+
 static void
 ub_resolve_callback(void* arg, int err, struct ub_result* ub_res)
 {
 	getdns_network_req *netreq = (getdns_network_req *) arg;
 	getdns_dns_req *dns_req = netreq->owner;
 
-	netreq->state = NET_REQ_FINISHED;
+	_getdns_netreq_change_state(netreq, NET_REQ_FINISHED);
 	if (err != 0) {
 		_getdns_call_user_callback(dns_req, NULL);
 		return;
 	}
 	/* parse */
-	if (getdns_apply_network_result(netreq, ub_res)) {
+	if (getdns_apply_network_result(netreq, ub_res->rcode,
+	    ub_res->answer_packet, ub_res->answer_len,
+	    (ub_res->secure ? 2 : ub_res->bogus ? 1 : 0), ub_res->why_bogus)) {
 		ub_resolve_free(ub_res);
 		_getdns_call_user_callback(dns_req, NULL);
 		return;
@@ -280,27 +282,134 @@ ub_resolve_callback(void* arg, int err, struct ub_result* ub_res)
 #endif
 
 
-getdns_return_t
-_getdns_submit_netreq(getdns_network_req *netreq)
+void _getdns_check_expired_pending_netreqs(
+    getdns_context *context, uint64_t *now_ms)
+{
+	getdns_network_req *first;
+
+	assert(context);
+
+	while (context->pending_netreqs.count) {
+		first = (getdns_network_req *)
+		    _getdns_rbtree_first(&context->pending_netreqs);
+
+		if (_getdns_ms_until_expiry2(first->owner->expires, now_ms) > 0)
+			break;
+
+		(void) _getdns_rbtree_delete(&context->pending_netreqs, first);
+		_getdns_netreq_change_state(first, NET_REQ_TIMED_OUT);
+		_getdns_check_dns_req_complete(first->owner);
+	}
+	first = context->pending_netreqs.count ? (getdns_network_req *)
+	    _getdns_rbtree_first(&context->pending_netreqs) : NULL;
+
+	if (first == context->first_pending_netreq ||
+	    (first && context->first_pending_netreq &&
+	     first->owner->expires == context->first_pending_netreq->owner->expires))
+		return; /* Nothing changed */
+
+	if (context->first_pending_netreq)
+		GETDNS_CLEAR_EVENT(  context->extension
+		                  , &context->pending_timeout_event);
+
+	if ((context->first_pending_netreq = first))
+		GETDNS_SCHEDULE_EVENT( context->extension, -1,
+		    _getdns_ms_until_expiry2(first->owner->expires, now_ms),
+		    &context->pending_timeout_event);
+}
+
+void
+_getdns_netreq_change_state(
+    getdns_network_req *netreq, network_req_state new_state)
+{
+	getdns_context *context;
+	uint64_t now_ms;
+	getdns_network_req *prev;
+
+	if (!netreq)
+		return;
+
+	if (!netreq->owner->is_dns_request) {
+		netreq->state = new_state;
+		return;
+	}
+
+	context = netreq->owner->context;
+
+	if (netreq->state != NET_REQ_IN_FLIGHT) {
+		if (new_state == NET_REQ_IN_FLIGHT)
+			context->netreqs_in_flight += 1;
+		netreq->state = new_state;
+		return;
+	}
+	if (new_state == NET_REQ_IN_FLIGHT) /* No change */
+		return;
+	netreq->state = new_state;
+	context->netreqs_in_flight -= 1;
+
+	now_ms = 0;
+	prev = NULL;
+	while (context->pending_netreqs.count > 0 &&
+	    (  context->limit_outstanding_queries > context->netreqs_in_flight
+	    || context->limit_outstanding_queries == 0 ))  {
+
+		getdns_network_req *first = (getdns_network_req *)
+		    _getdns_rbtree_first(&context->pending_netreqs);
+		
+		/* To prevent loops due to _getdns_submit_netreq re-inserting
+		 * because of errno == EMFILE
+		 */
+		if (first == prev)
+			break;
+		else
+			prev = first;
+
+		(void) _getdns_rbtree_delete(&context->pending_netreqs, first);
+		(void) _getdns_submit_netreq(first, &now_ms);
+	}
+}
+
+int
+_getdns_submit_netreq(getdns_network_req *netreq, uint64_t *now_ms)
 {
 	getdns_return_t r;
 	getdns_dns_req *dns_req = netreq->owner;
+	getdns_context *context = dns_req->context;
 	char name[1024];
+	int dnsreq_freed = 0;
+#ifdef HAVE_LIBUNBOUND
+	int ub_resolve_r;
+#endif
 
+	if (context->limit_outstanding_queries > 0 &&
+	    context->netreqs_in_flight >= context->limit_outstanding_queries) {
+
+		netreq->node.key = netreq;
+		if (_getdns_rbtree_insert(
+		    &context->pending_netreqs, &netreq->node)) {
+			
+			_getdns_check_expired_pending_netreqs(context, now_ms);
+			return GETDNS_RETURN_GOOD;
+		}
+	}
+	_getdns_netreq_change_state(netreq, NET_REQ_IN_FLIGHT);
+
+	debug_req("Submitting ", netreq);
 
 #ifdef STUB_NATIVE_DNSSEC
 # ifdef DNSSEC_ROADBLOCK_AVOIDANCE
 
-	if ((dns_req->context->resolution_type == GETDNS_RESOLUTION_RECURSING
+	if ((context->resolution_type == GETDNS_RESOLUTION_RECURSING
 	    && !dns_req->dnssec_roadblock_avoidance)
 	    ||  dns_req->avoid_dnssec_roadblocks) {
 # else
-	if ( dns_req->context->resolution_type == GETDNS_RESOLUTION_RECURSING) {
+	if ( context->resolution_type == GETDNS_RESOLUTION_RECURSING) {
 # endif
 #else
-	if ( dns_req->context->resolution_type == GETDNS_RESOLUTION_RECURSING
+	if ( context->resolution_type == GETDNS_RESOLUTION_RECURSING
 	    || dns_req->dnssec_return_status
 	    || dns_req->dnssec_return_only_secure
+	    || dns_req->dnssec_return_all_statuses
 	    || dns_req->dnssec_return_validation_chain) {
 #endif
 		/* schedule the timeout */
@@ -308,39 +417,142 @@ _getdns_submit_netreq(getdns_network_req *netreq)
 			dns_req->timeout.userarg    = dns_req;
 			dns_req->timeout.read_cb    = NULL;
 			dns_req->timeout.write_cb   = NULL;
-			dns_req->timeout.timeout_cb = ub_resolve_timeout;
+			dns_req->timeout.timeout_cb =
+			    (getdns_eventloop_callback)
+			    _getdns_context_request_timed_out;
 			dns_req->timeout.ev         = NULL;
 			if ((r = dns_req->loop->vmt->schedule(dns_req->loop, -1,
-			    dns_req->context->timeout, &dns_req->timeout)))
+			    _getdns_ms_until_expiry2(dns_req->expires, now_ms),
+			    &dns_req->timeout)))
 				return r;
 		}
 		(void) gldns_wire2str_dname_buf(dns_req->name,
 		    dns_req->name_len, name, sizeof(name));
 
 #ifdef HAVE_LIBUNBOUND
-		return ub_resolve_async(dns_req->context->unbound_ctx,
-		    name, netreq->request_type, netreq->owner->request_class,
-		    netreq, ub_resolve_callback, &(netreq->unbound_id)) ?
-		    GETDNS_RETURN_GENERIC_ERROR : GETDNS_RETURN_GOOD;
+		dns_req->freed = &dnsreq_freed;
+#ifdef HAVE_UNBOUND_EVENT_API
+		if (_getdns_ub_loop_enabled(&context->ub_loop))
+			ub_resolve_r = ub_resolve_event(context->unbound_ctx,
+			    name, netreq->request_type, dns_req->request_class,
+			    netreq, ub_resolve_event_callback, &(netreq->unbound_id)) ?
+			    GETDNS_RETURN_GENERIC_ERROR : GETDNS_RETURN_GOOD;
+		else
+#endif
+			ub_resolve_r = ub_resolve_async(context->unbound_ctx,
+			    name, netreq->request_type, dns_req->request_class,
+			    netreq, ub_resolve_callback, &(netreq->unbound_id)) ?
+			    GETDNS_RETURN_GENERIC_ERROR : GETDNS_RETURN_GOOD;
+		if (dnsreq_freed)
+			return DNS_REQ_FINISHED;
+		dns_req->freed = NULL;
+		return ub_resolve_r ? GETDNS_RETURN_GENERIC_ERROR : GETDNS_RETURN_GOOD;
 #else
 		return GETDNS_RETURN_NOT_IMPLEMENTED;
 #endif
 	}
 	/* Submit with stub resolver */
-	return _getdns_submit_stub_request(netreq);
+	dns_req->freed = &dnsreq_freed;
+	r = _getdns_submit_stub_request(netreq, now_ms);
+	if (dnsreq_freed)
+		return DNS_REQ_FINISHED;
+	dns_req->freed = NULL;
+	return r;
 }
 
+
+/**
+ * structure used by validate_extensions() to check extension formats
+ */
+typedef struct getdns_extension_format
+{
+	char *extstring;
+	getdns_data_type exttype;
+	int implemented;
+} getdns_extension_format;
+
+static int
+extformatcmp(const void *a, const void *b)
+{
+	return strcmp(((getdns_extension_format *) a)->extstring,
+	    ((getdns_extension_format *) b)->extstring);
+}
+
+/*---------------------------------------- validate_extensions */
+static getdns_return_t
+validate_extensions(struct getdns_dict * extensions)
+{
+	/**
+	  * this is a comprehensive list of extensions and their data types
+	  * used by validate_extensions()
+	  * The list has to be in sorted order for bsearch lookup in function
+	  * validate_extensions.
+	  */
+	static getdns_extension_format extformats[] = {
+		{"add_opt_parameters"            , t_dict, 1},
+		{"add_warning_for_bad_dns"       , t_int , 1},
+		{"dnssec_return_all_statuses"    , t_int , 1},
+		{"dnssec_return_full_validation_chain", t_int , 1},
+		{"dnssec_return_only_secure"     , t_int , 1},
+		{"dnssec_return_status"          , t_int , 1},
+		{"dnssec_return_validation_chain", t_int , 1},
+		{"dnssec_roadblock_avoidance"    , t_int ,
+#if defined(DNSSEC_ROADBLOCK_AVOIDANCE) && defined(HAVE_LIBUNBOUND)
+							   1},
+#else
+							   0},
+#endif
+		{"edns_cookies"                  , t_int ,
+#ifdef EDNS_COOKIES
+							   1},
+#else
+							   0},
+#endif
+		{"header"                        , t_dict, 1},
+		{"return_api_information"        , t_int , 1},
+		{"return_both_v4_and_v6"         , t_int , 1},
+		{"return_call_reporting"         , t_int , 1},
+		{"specify_class"                 , t_int , 1},
+	};
+
+	struct getdns_dict_item *item;
+	getdns_extension_format *extformat;
+
+	if (extensions)
+		RBTREE_FOR(item, struct getdns_dict_item *,
+		    &(extensions->root)) {
+
+			getdns_extension_format key;
+			key.extstring = (char *) item->node.key;
+			extformat = bsearch(&key, extformats,
+			    sizeof(extformats) /
+			    sizeof(getdns_extension_format),
+			    sizeof(getdns_extension_format), extformatcmp);
+			if (!extformat)
+				return GETDNS_RETURN_NO_SUCH_EXTENSION;
+
+			if (!extformat->implemented)
+				return GETDNS_RETURN_NOT_IMPLEMENTED;
+
+			if (item->i.dtype != extformat->exttype)
+				return GETDNS_RETURN_EXTENSION_MISFORMAT;
+		}
+	return GETDNS_RETURN_GOOD;
+}				/* _getdns_validate_extensions */
+
+
 static getdns_return_t
 getdns_general_ns(getdns_context *context, getdns_eventloop *loop,
     const char *name, uint16_t request_type, getdns_dict *extensions,
-    void *userarg, getdns_dns_req **dnsreq_p,
+    void *userarg, getdns_network_req **return_netreq_p,
     getdns_callback_t callbackfn, internal_cb_t internal_cb, int usenamespaces)
 {
-	getdns_return_t r = GETDNS_RETURN_GOOD;
+	int r = GETDNS_RETURN_GOOD;
 	getdns_network_req *netreq, **netreq_p;
 	getdns_dns_req *req;
 	getdns_dict *localnames_response;
 	size_t i;
+	uint64_t now_ms = 0;
 
 	if (!context || !name || (!callbackfn && !internal_cb))
 		return GETDNS_RETURN_INVALID_PARAMETER;
@@ -348,7 +560,7 @@ getdns_general_ns(getdns_context *context, getdns_eventloop *loop,
 	if ((r = _getdns_validate_dname(name)))
 		return r;
 
-	if (extensions && (r = _getdns_validate_extensions(extensions)))
+	if (extensions && (r = validate_extensions(extensions)))
 		return r;
 
 	/* Set up the context assuming we won't use the specified namespaces.
@@ -357,15 +569,17 @@ getdns_general_ns(getdns_context *context, getdns_eventloop *loop,
 		return r;
 
 	/* create the request */
-	if (!(req = _getdns_dns_req_new(context, loop, name, request_type, extensions)))
+	if (!(req = _getdns_dns_req_new(
+	    context, loop, name, request_type, extensions, &now_ms)))
 		return GETDNS_RETURN_MEMORY_ERROR;
 
 	req->user_pointer = userarg;
 	req->user_callback = callbackfn;
 	req->internal_cb = internal_cb;
+	req->is_sync_request = loop == &context->sync_eventloop.loop;
 
-	if (dnsreq_p)
-		*dnsreq_p = req;
+	if (return_netreq_p)
+		*return_netreq_p = req->netreqs[0];
 
 	_getdns_context_track_outbound_request(req);
 
@@ -373,19 +587,50 @@ getdns_general_ns(getdns_context *context, getdns_eventloop *loop,
 		/* issue all network requests */
 		for ( netreq_p = req->netreqs
 		    ; !r && (netreq = *netreq_p)
-		    ; netreq_p++)
-			r = _getdns_submit_netreq(netreq);
+		    ; netreq_p++) {
+			if ((r = _getdns_submit_netreq(netreq, &now_ms))) {
+				if (r == DNS_REQ_FINISHED) {
+					if (return_netreq_p)
+						*return_netreq_p = NULL;
+					return GETDNS_RETURN_GOOD;
+				}
+				_getdns_netreq_change_state(netreq, NET_REQ_ERRORED);
+			}
+		}
 
 	else for (i = 0; i < context->namespace_count; i++) {
 		if (context->namespaces[i] == GETDNS_NAMESPACE_LOCALNAMES) {
 
 			if (!(r = _getdns_context_local_namespace_resolve(
 			    req, &localnames_response))) {
-
+				req->is_dns_request = 0;
 				_getdns_call_user_callback
 				    ( req, localnames_response);
 				break;
 			}
+#ifdef HAVE_MDNS_SUPPORT
+		} else if (context->namespaces[i] == GETDNS_NAMESPACE_MDNS) {
+			/* Check whether the name belongs in the MDNS space */
+			if (!(r = _getdns_mdns_namespace_check(req)))
+			{
+				req->is_dns_request = 0;
+				// Submit the query to the MDNS transport.
+				for (netreq_p = req->netreqs
+					; !r && (netreq = *netreq_p)
+					; netreq_p++) {
+					if ((r = _getdns_submit_mdns_request(netreq))) {
+						if (r == DNS_REQ_FINISHED) {
+							if (return_netreq_p)
+								*return_netreq_p = NULL;
+							return GETDNS_RETURN_GOOD;
+						}
+						_getdns_netreq_change_state(netreq, NET_REQ_ERRORED);
+					}
+				}
+				/* Stop processing more namespaces, since there was a match */
+				break;
+			}
+#endif /* HAVE_MDNS_SUPPORT */
 		} else if (context->namespaces[i] == GETDNS_NAMESPACE_DNS) {
 
 			/* TODO: We will get a good return code here even if
@@ -395,14 +640,21 @@ getdns_general_ns(getdns_context *context, getdns_eventloop *loop,
 			r = GETDNS_RETURN_GOOD;
 			for ( netreq_p = req->netreqs
 			    ; !r && (netreq = *netreq_p)
-			    ; netreq_p++)
-				r = _getdns_submit_netreq(netreq);
+			    ; netreq_p++) {
+				if ((r = _getdns_submit_netreq(netreq, &now_ms))) {
+					if (r == DNS_REQ_FINISHED) {
+						if (return_netreq_p)
+							*return_netreq_p = NULL;
+						return GETDNS_RETURN_GOOD;
+					}
+					_getdns_netreq_change_state(netreq, NET_REQ_ERRORED);
+				}
+			}
 			break;
 		} else
 			r = GETDNS_RETURN_BAD_CONTEXT;
 	}
-
-	if (r != 0) {
+	if (r > 0) { /* i.e. r != GETDNS_RETURN_GOOD && r != DNS_REQ_FINISHED */
 		/* clean up the request */
 		_getdns_context_clear_outbound_request(req);
 		_getdns_dns_req_free(req);
@@ -414,12 +666,12 @@ getdns_general_ns(getdns_context *context, getdns_eventloop *loop,
 getdns_return_t
 _getdns_general_loop(getdns_context *context, getdns_eventloop *loop,
     const char *name, uint16_t request_type, getdns_dict *extensions,
-    void *userarg, getdns_dns_req **dnsreq_p,
+    void *userarg, getdns_network_req **netreq_p,
     getdns_callback_t callback, internal_cb_t internal_cb)
 {
 	return getdns_general_ns(context, loop,
 	    name, request_type, extensions,
-	    userarg, dnsreq_p, callback, internal_cb, 0);
+	    userarg, netreq_p, callback, internal_cb, 0);
 
 }				/* getdns_general_loop */
 
@@ -431,7 +683,7 @@ _getdns_address_loop(getdns_context *context, getdns_eventloop *loop,
 	getdns_dict *my_extensions = extensions;
 	getdns_return_t r;
 	uint32_t value;
-	getdns_dns_req *dnsreq = NULL;
+	getdns_network_req *netreq = NULL;
 
 	if (!my_extensions) {
 		if (!(my_extensions=getdns_dict_create_with_context(context)))
@@ -447,9 +699,9 @@ _getdns_address_loop(getdns_context *context, getdns_eventloop *loop,
 
 	r = getdns_general_ns(context, loop,
 	    name, GETDNS_RRTYPE_AAAA, my_extensions,
-	    userarg, &dnsreq, callback, NULL, 1);
-	if (dnsreq && transaction_id)
-		*transaction_id = dnsreq->trans_id;
+	    userarg, &netreq, callback, NULL, 1);
+	if (netreq && transaction_id)
+		*transaction_id = netreq->owner->trans_id;
 
 	if (my_extensions != extensions)
 		getdns_dict_destroy(my_extensions);
@@ -467,7 +719,7 @@ _getdns_hostname_loop(getdns_context *context, getdns_eventloop *loop,
 	uint16_t req_type;
 	char name[1024];
 	getdns_return_t retval;
-	getdns_dns_req *dnsreq = NULL;
+	getdns_network_req *netreq= NULL;
 
 	if ((retval =
 		getdns_dict_get_bindata(address, "address_data",
@@ -541,9 +793,9 @@ _getdns_hostname_loop(getdns_context *context, getdns_eventloop *loop,
 		return GETDNS_RETURN_INVALID_PARAMETER;
 	}
 	retval = _getdns_general_loop(context, loop, name, req_type,
-	    extensions, userarg, &dnsreq, callback, NULL);
-	if (dnsreq && transaction_id)
-		*transaction_id = dnsreq->trans_id;
+	    extensions, userarg, &netreq, callback, NULL);
+	if (netreq && transaction_id)
+		*transaction_id = netreq->owner->trans_id;
 	return retval;
 }				/* getdns_hostname_loop */
 
@@ -553,11 +805,11 @@ _getdns_service_loop(getdns_context *context, getdns_eventloop *loop,
     getdns_transaction_t * transaction_id, getdns_callback_t callback)
 {
 	getdns_return_t r;
-	getdns_dns_req *dnsreq = NULL;
+	getdns_network_req *netreq = NULL;
 	r = getdns_general_ns(context, loop, name, GETDNS_RRTYPE_SRV,
-	    extensions, userarg, &dnsreq, callback, NULL, 1);
-	if (dnsreq && transaction_id)
-		*transaction_id = dnsreq->trans_id;
+	    extensions, userarg, &netreq, callback, NULL, 1);
+	if (netreq && transaction_id)
+		*transaction_id = netreq->owner->trans_id;
 	return r;
 }				/* getdns_service_loop */
 
@@ -568,17 +820,17 @@ getdns_return_t
 getdns_general(getdns_context *context,
     const char *name, uint16_t request_type, getdns_dict *extensions,
     void *userarg, getdns_transaction_t * transaction_id,
-    getdns_callback_t callback)
+    getdns_callback_t callbackfn)
 {
 	getdns_return_t r;
-	getdns_dns_req *dnsreq = NULL;
+	getdns_network_req *netreq = NULL;
 
 	if (!context) return GETDNS_RETURN_INVALID_PARAMETER;
 	r = _getdns_general_loop(context, context->extension,
 	    name, request_type, extensions,
-	    userarg, &dnsreq, callback, NULL);
-	if (dnsreq && transaction_id)
-		*transaction_id = dnsreq->trans_id;
+	    userarg, &netreq, callbackfn, NULL);
+	if (netreq && transaction_id)
+		*transaction_id = netreq->owner->trans_id;
 	return r;
 }				/* getdns_general */
 
@@ -589,12 +841,12 @@ getdns_general(getdns_context *context,
 getdns_return_t
 getdns_address(getdns_context *context,
     const char *name, getdns_dict *extensions, void *userarg,
-    getdns_transaction_t *transaction_id, getdns_callback_t callback)
+    getdns_transaction_t *transaction_id, getdns_callback_t callbackfn)
 {
 	if (!context) return GETDNS_RETURN_INVALID_PARAMETER;
 	return _getdns_address_loop(context, context->extension,
 	    name, extensions, userarg,
-	    transaction_id, callback);
+	    transaction_id, callbackfn);
 } /* getdns_address */
 
 /*
@@ -604,11 +856,11 @@ getdns_address(getdns_context *context,
 getdns_return_t
 getdns_hostname(getdns_context *context,
     getdns_dict *address, getdns_dict *extensions, void *userarg,
-    getdns_transaction_t *transaction_id, getdns_callback_t callback)
+    getdns_transaction_t *transaction_id, getdns_callback_t callbackfn)
 {
 	if (!context) return GETDNS_RETURN_INVALID_PARAMETER;
 	return _getdns_hostname_loop(context, context->extension,
-	    address, extensions, userarg, transaction_id, callback);
+	    address, extensions, userarg, transaction_id, callbackfn);
 }				/* getdns_hostname */
 
 /*
@@ -618,11 +870,11 @@ getdns_hostname(getdns_context *context,
 getdns_return_t
 getdns_service(getdns_context *context,
     const char *name, getdns_dict *extensions, void *userarg,
-    getdns_transaction_t *transaction_id, getdns_callback_t callback)
+    getdns_transaction_t *transaction_id, getdns_callback_t callbackfn)
 {
 	if (!context) return GETDNS_RETURN_INVALID_PARAMETER;
 	return _getdns_service_loop(context, context->extension,
-	    name, extensions, userarg, transaction_id, callback);
+	    name, extensions, userarg, transaction_id, callbackfn);
 }				/* getdns_service */
 
 /* getdns_general.c */
