@@ -47,6 +47,8 @@ static gldns_lookup_table gldns_algorithms_data[] = {
 	{ GLDNS_ECC_GOST, "ECC-GOST"},
 	{ GLDNS_ECDSAP256SHA256, "ECDSAP256SHA256"},
 	{ GLDNS_ECDSAP384SHA384, "ECDSAP384SHA384"},
+	{ GLDNS_ED25519, "ED25519"},
+	{ GLDNS_ED448, "ED448"},
 	{ GLDNS_INDIRECT, "INDIRECT" },
 	{ GLDNS_PRIVATEDNS, "PRIVATEDNS" },
 	{ GLDNS_PRIVATEOID, "PRIVATEOID" },
@@ -166,6 +168,7 @@ static gldns_lookup_table gldns_edns_options_data[] = {
 	{ 7, "N3U" },
 	{ 8, "edns-client-subnet" },
 	{ 11, "edns-tcp-keepalive"},
+	{ 12, "Padding" },
 	{ 0, NULL}
 };
 gldns_lookup_table* gldns_edns_options = gldns_edns_options_data;
@@ -192,14 +195,22 @@ char* gldns_wire2str_type(uint16_t rrtype)
 {
 	char buf[16];
 	gldns_wire2str_type_buf(rrtype, buf, sizeof(buf));
+#ifndef USE_WINSOCK
 	return strdup(buf);
+#else
+	return _strdup(buf);
+#endif
 }
 
 char* gldns_wire2str_class(uint16_t rrclass)
 {
 	char buf[16];
 	gldns_wire2str_class_buf(rrclass, buf, sizeof(buf));
+#ifndef USE_WINSOCK
 	return strdup(buf);
+#else
+	return _strdup(buf);
+#endif
 }
 
 char* gldns_wire2str_dname(uint8_t* dname, size_t dname_len)
@@ -215,7 +226,11 @@ char* gldns_wire2str_rcode(int rcode)
 {
 	char buf[16];
 	gldns_wire2str_rcode_buf(rcode, buf, sizeof(buf));
+#ifndef USE_WINSOCK
 	return strdup(buf);
+#else
+	return _strdup(buf);
+#endif
 }
 
 int gldns_wire2str_pkt_buf(uint8_t* d, size_t dlen, char* s, size_t slen)
@@ -667,7 +682,7 @@ int gldns_wire2str_rdata_scan(uint8_t** d, size_t* dlen, char** s,
 	uint8_t* origd = *d;
 	char* origs = *s;
 	size_t origdlen = *dlen, origslen = *slen;
-	uint16_t r_cnt, r_max;
+	size_t r_cnt, r_max;
 	gldns_rdf_type rdftype;
 	int w = 0, n;
 
@@ -698,6 +713,9 @@ int gldns_wire2str_rdata_scan(uint8_t** d, size_t* dlen, char** s,
 		}
 		w += n;
 	}
+	if(*dlen != 0) {
+		goto failed;
+	}
 	return w;
 }
 
@@ -723,7 +741,7 @@ static int dname_char_print(char** s, size_t* slen, uint8_t c)
 {
 	if(c == '.' || c == ';' || c == '(' || c == ')' || c == '\\')
 		return gldns_str_print(s, slen, "\\%c", c);
-	else if(!(isascii((int)c) && isgraph((int)c)))
+	else if(!(isascii((unsigned char)c) && isgraph((unsigned char)c)))
 		return gldns_str_print(s, slen, "\\%03u", (unsigned)c);
 	/* plain printout */
 	if(*slen) {
@@ -785,8 +803,9 @@ int gldns_wire2str_dname_scan(uint8_t** d, size_t* dlen, char** s, size_t* slen,
 		}
 
 		/* spool label characters, end with '.' */
-		if(in_buf && *dlen < labellen) labellen = *dlen;
-		else if(!in_buf && pos+labellen > pkt+pktlen)
+		if(in_buf && *dlen < (size_t)labellen)
+			labellen = (uint8_t)*dlen;
+		else if(!in_buf && pos+(size_t)labellen > pkt+pktlen)
 			labellen = (uint8_t)(pkt + pktlen - pos);
 		for(i=0; i<(unsigned)labellen; i++) {
 			w += dname_char_print(s, slen, *pos++);
@@ -1065,7 +1084,7 @@ int gldns_wire2str_aaaa_scan(uint8_t** d, size_t* dl, char** s, size_t* sl)
 /** printout escaped TYPE_STR character */
 static int str_char_print(char** s, size_t* sl, uint8_t c)
 {
-	if(isprint((int)c) || c == '\t') {
+	if(isprint((unsigned char)c) || c == '\t') {
 		if(c == '\"' || c == '\\')
 			return gldns_str_print(s, sl, "\\%c", c);
 		if(*sl) {
@@ -1626,7 +1645,7 @@ int gldns_wire2str_tag_scan(uint8_t** d, size_t* dl, char** s, size_t* sl)
 	if(*dl < 1+n)
 		return -1;
 	for(i=0; i<n; i++)
-		if(!isalnum((int)(*d)[i]))
+		if(!isalnum((unsigned char)(*d)[i]))
 			return -1;
 	for(i=0; i<n; i++)
 		w += gldns_str_print(s, sl, "%c", (char)(*d)[i]);
@@ -1714,7 +1733,7 @@ int gldns_wire2str_edns_nsid_print(char** s, size_t* sl, uint8_t* data,
 	size_t i, printed=0;
 	w += print_hex_buf(s, sl, data, len);
 	for(i=0; i<len; i++) {
-		if(isprint((int)data[i]) || data[i] == '\t') {
+		if(isprint((unsigned char)data[i]) || data[i] == '\t') {
 			if(!printed) {
 				w += gldns_str_print(s, sl, " (");
 				printed = 1;
@@ -1883,7 +1902,10 @@ int gldns_wire2str_edns_option_print(char** s, size_t* sl,
 		break;
 	 case GLDNS_EDNS_KEEPALIVE:
 		w += gldns_wire2str_edns_keepalive_print(s, sl, optdata, optlen);
-		break; 
+		break;
+	case GLDNS_EDNS_PADDING:
+		w += print_hex_buf(s, sl, optdata, optlen);
+		break;
 	default:
 		/* unknown option code */
 		w += print_hex_buf(s, sl, optdata, optlen);
@@ -1976,10 +1998,10 @@ int gldns_wire2str_edns_scan(uint8_t** data, size_t* data_len, char** str,
 	w += gldns_str_print(str, str_len, " ; udp: %u", (unsigned)udpsize);
 
 	if(rdatalen) {
-		if(*data_len < rdatalen) {
+		if((size_t)*data_len < rdatalen) {
 			w += gldns_str_print(str, str_len,
 				" ; Error EDNS rdata too short; ");
-			rdatalen = *data_len;
+			rdatalen = (uint16_t)*data_len;
 		}
 		w += print_edns_opts(str, str_len, *data, rdatalen);
 		(*data) += rdatalen;
