@@ -54,6 +54,11 @@
 #include <openssl/x509.h>
 #include <string.h>
 #include "context.h"
+#include "util-internal.h"
+
+#if OPENSSL_VERSION_NUMBER < 0x10100000 || defined(HAVE_LIBRESSL)
+#define X509_STORE_CTX_get0_untrusted(store) store->untrusted
+#endif
 
 /* we only support sha256 at the moment.  adding support for another
    digest is more complex than just adding another entry here. in
@@ -92,14 +97,14 @@ getdns_dict* getdns_pubkey_pin_create_from_string(
 	const char* str)
 {
 	BIO *bio = NULL;
-	int i;
+	size_t i;
 	uint8_t buf[SHA256_DIGEST_LENGTH];
 	char inbuf[B64_ENCODED_SHA256_LENGTH + 1];
 	getdns_bindata value = { .size = SHA256_DIGEST_LENGTH, .data = buf };
 	getdns_dict* out = NULL;
 	
 	/* we only do sha256 right now, make sure this is well-formed */
-	if (strncmp(PIN_PREFIX, str, PIN_PREFIX_LENGTH))
+	if (!str || strncmp(PIN_PREFIX, str, PIN_PREFIX_LENGTH))
 		return NULL;
 	for (i = PIN_PREFIX_LENGTH; i < PIN_PREFIX_LENGTH + B64_ENCODED_SHA256_LENGTH - 1; i++)
 		if (!((str[i] >= 'a' && str[i] <= 'z') ||
@@ -156,11 +161,9 @@ getdns_dict* getdns_pubkey_pin_create_from_string(
 */
 
 #define PKP_SC_ERR(e) { \
-       err.size = sizeof(e); \
-       err.data = (uint8_t*)e; \
        if (errorlist) \
-	       getdns_list_set_bindata(errorlist, \
-				       preverrs + errorcount, &err); \
+	       _getdns_list_append_const_bindata(errorlist, \
+				       sizeof(e), e); \
        errorcount++; \
 	}
 #define PKP_SC_HARDERR(e, val) { \
@@ -170,15 +173,10 @@ getdns_return_t getdns_pubkey_pinset_sanity_check(
 	const getdns_list* pinset,
 	getdns_list* errorlist)
 {
-	size_t errorcount = 0, preverrs = 0, pins = 0, i;
-	getdns_bindata err;
+	size_t errorcount = 0, pins = 0, i;
 	getdns_dict * pin;
 	getdns_bindata * data;
 
-	if (errorlist)
-		if (getdns_list_get_length(errorlist, &preverrs))
-			return GETDNS_RETURN_INVALID_PARAMETER;
-	
 	if (getdns_list_get_length(pinset, &pins))
 		PKP_SC_HARDERR("Can't get length of pinset",
 			       GETDNS_RETURN_INVALID_PARAMETER);
@@ -281,7 +279,6 @@ _getdns_get_pubkey_pinset_list(getdns_context *ctx,
 	uint8_t buf[SHA256_DIGEST_LENGTH];
 	getdns_bindata value = { .size = SHA256_DIGEST_LENGTH, .data = buf };
 	getdns_dict *pin = NULL;
-	size_t idx = 0;
 
 	if (out == NULL)
 		return GETDNS_RETURN_MEMORY_ERROR;
@@ -296,9 +293,8 @@ _getdns_get_pubkey_pinset_list(getdns_context *ctx,
 		memcpy(buf, pinset_in->pin, sizeof(buf));
 		if (r = getdns_dict_set_bindata(pin, "value", &value), r)
 			goto fail;
-		if (r = getdns_list_set_dict(out, idx++, pin), r)
+		if (r = _getdns_list_append_this_dict(out, pin), r)
 			goto fail;
-		getdns_dict_destroy(pin);
 		pin = NULL;
 		pinset_in = pinset_in->next;
 	}
@@ -318,15 +314,27 @@ _getdns_get_pubkey_pinset_list(getdns_context *ctx,
    see doc/HOWTO/proxy_certificates.txt as an example
 */
 static int
-_get_ssl_getdns_upstream_idx()
+#if OPENSSL_VERSION_NUMBER < 0x10100000 || defined(HAVE_LIBRESSL)
+_get_ssl_getdns_upstream_idx(void)
+#else
+_get_ssl_getdns_upstream_idx(X509_STORE *store)
+#endif
 {
 	static volatile int idx = -1;
 	if (idx < 0) {
+#if OPENSSL_VERSION_NUMBER < 0x10100000 || defined(HAVE_LIBRESSL)
 		CRYPTO_w_lock(CRYPTO_LOCK_X509_STORE);
+#else
+		X509_STORE_lock(store);
+#endif
 		if (idx < 0)
 			idx = SSL_get_ex_new_index(0, "associated getdns upstream",
 						   NULL,NULL,NULL);
+#if OPENSSL_VERSION_NUMBER < 0x10100000 || defined(HAVE_LIBRESSL)
 		CRYPTO_w_unlock(CRYPTO_LOCK_X509_STORE);
+#else
+		X509_STORE_unlock(store);
+#endif
 	}
 	return idx;
 }
@@ -334,7 +342,11 @@ _get_ssl_getdns_upstream_idx()
 getdns_upstream*
 _getdns_upstream_from_x509_store(X509_STORE_CTX *store)
 {
+#if OPENSSL_VERSION_NUMBER < 0x10100000 || defined(HAVE_LIBRESSL)
 	int uidx = _get_ssl_getdns_upstream_idx();
+#else
+	int uidx = _get_ssl_getdns_upstream_idx(X509_STORE_CTX_get0_store(store));
+#endif
 	int sslidx = SSL_get_ex_data_X509_STORE_CTX_idx();
 	const SSL *ssl;
 
@@ -352,7 +364,11 @@ getdns_return_t
 _getdns_associate_upstream_with_SSL(SSL *ssl,
 				    getdns_upstream *upstream)
 {
+#if OPENSSL_VERSION_NUMBER < 0x10100000 || defined(HAVE_LIBRESSL)
 	int uidx = _get_ssl_getdns_upstream_idx();
+#else
+	int uidx = _get_ssl_getdns_upstream_idx(SSL_CTX_get_cert_store(SSL_get_SSL_CTX(ssl)));
+#endif
 	if (SSL_set_ex_data(ssl, uidx, upstream))
 		return GETDNS_RETURN_GOOD;
 	else
@@ -366,10 +382,10 @@ _getdns_verify_pinset_match(const sha256_pin_t *pinset,
 			    X509_STORE_CTX *store)
 {
 	getdns_return_t ret = GETDNS_RETURN_GENERIC_ERROR;
-	X509 *x;
+	X509 *x, *prev = NULL;
 	int i, len;
 	unsigned char raw[4096];
-	unsigned char *next = raw;
+	unsigned char *next;
 	unsigned char buf[sizeof(pinset->pin)];
 	const sha256_pin_t *p;
 
@@ -391,38 +407,45 @@ _getdns_verify_pinset_match(const sha256_pin_t *pinset,
 
 	/* TODO: how do we handle raw public keys? */
 
-	for (i = 0; i < sk_X509_num(store->untrusted); i++) {
-		if (i > 0) {
-		/* TODO: how do we ensure that the certificates in
-		 * each stage appropriately sign the previous one?
-		 * for now, to be safe, we only examine the end-entity
-		 * cert: */
-			return GETDNS_RETURN_GENERIC_ERROR;
-		}
+	for (i = 0; i < sk_X509_num(X509_STORE_CTX_get0_untrusted(store)); i++, prev = x) {
 
-		x = sk_X509_value(store->untrusted, i);
-		if (x->cert_info == NULL)
-			continue;
+		x = sk_X509_value(X509_STORE_CTX_get0_untrusted(store), i);
 #if defined(STUB_DEBUG) && STUB_DEBUG
-		DEBUG_STUB("--- %s: name of cert %d:\n", __FUNCTION__, i);
-		if (x->cert_info->subject != NULL)
-			X509_NAME_print_ex_fp(stderr, x->cert_info->subject, 4, XN_FLAG_ONELINE);
+		DEBUG_STUB("%s %-35s: Name of cert: %d ",
+		           STUB_DEBUG_SETUP_TLS, __FUNC__, i);
+		X509_NAME_print_ex_fp(stderr, X509_get_subject_name(x), 1, XN_FLAG_ONELINE);
 		fprintf(stderr, "\n");
 #endif
-		if (x->cert_info->key == NULL)
-			continue;
+		if (i > 0) {
+		/* we ensure that "prev" is signed by "x" */
+			EVP_PKEY *pkey = X509_get_pubkey(x);
+			int verified;
+			if (!pkey) {
+				DEBUG_STUB("%s %-35s: Could not get pubkey from cert %d (%p)\n",
+					   STUB_DEBUG_SETUP_TLS, __FUNC__, i, (void*)x);
+				return GETDNS_RETURN_GENERIC_ERROR;
+			}
+			verified = X509_verify(prev, pkey);
+			EVP_PKEY_free(pkey);
+			if (!verified) {
+				DEBUG_STUB("%s %-35s: cert %d (%p) was not signed by cert %d\n",
+					   STUB_DEBUG_SETUP_TLS, __FUNC__, i-1, (void*)prev, i);
+				return GETDNS_RETURN_GENERIC_ERROR;
+			}
+		}
 
 		/* digest the cert with sha256 */
 		len = i2d_X509_PUBKEY(X509_get_X509_PUBKEY(x), NULL);
-		if (len > sizeof(raw)) {
-			DEBUG_STUB("--- %s: pubkey %d is larger than %ld octets\n",
-				   __FUNCTION__, i, sizeof(raw));
+		if (len > (int)sizeof(raw)) {
+			DEBUG_STUB("%s %-35s: Pubkey %d is larger than "PRIsz" octets\n",
+			           STUB_DEBUG_SETUP_TLS, __FUNC__, i, sizeof(raw));
 			continue;
 		}
+		next = raw;
 		i2d_X509_PUBKEY(X509_get_X509_PUBKEY(x), &next);
 		if (next - raw != len) {
-			DEBUG_STUB("--- %s: pubkey %d claimed it needed %d octets, really needed %ld\n",
-				   __FUNCTION__, i, len, next - raw);
+			DEBUG_STUB("%s %-35s: Pubkey %d claimed it needed %d octets, really needed "PRIsz"\n",
+			           STUB_DEBUG_SETUP_TLS, __FUNC__, i, len, next - raw);
 			continue;
 		}
 		SHA256(raw, len, buf);
@@ -430,12 +453,12 @@ _getdns_verify_pinset_match(const sha256_pin_t *pinset,
 		/* compare it */
 		for (p = pinset; p; p = p->next)
 			if (0 == memcmp(buf, p->pin, sizeof(p->pin))) {
-				DEBUG_STUB("--- %s: pubkey %d matched pin %p (%ld)!\n",
-					   __FUNCTION__, i, p, sizeof(p->pin));
+				DEBUG_STUB("%s %-35s: Pubkey %d matched pin %p ("PRIsz")\n",
+					   STUB_DEBUG_SETUP_TLS, __FUNC__, i, (void*)p, sizeof(p->pin));
 				return GETDNS_RETURN_GOOD;
 			} else
-				DEBUG_STUB("--- %s: pubkey %d did not match pin %p!\n",
-					   __FUNCTION__, i, p);
+				DEBUG_STUB("%s %-35s: Pubkey %d did not match pin %p\n",
+					   STUB_DEBUG_SETUP_TLS, __FUNC__, i, (void*)p);
 	}
 
 	return ret;
