@@ -42,6 +42,8 @@
 #include "gldns/pkthdr.h"
 #include "dict.h"
 #include "debug.h"
+#include "convert.h"
+#include "general.h"
 
 /* MAXIMUM_TSIG_SPACE = TSIG name      (dname)    : 256
  *                      TSIG type      (uint16_t) :   2
@@ -64,40 +66,40 @@
 
 getdns_dict  dnssec_ok_checking_disabled_spc = {
 	{ RBTREE_NULL, 0, (int (*)(const void *, const void *)) strcmp },
-	{ 0 }
+	{ NULL, {{ NULL, NULL, NULL }}}
 };
 getdns_dict *dnssec_ok_checking_disabled = &dnssec_ok_checking_disabled_spc;
 
 getdns_dict  dnssec_ok_checking_disabled_roadblock_avoidance_spc = {
 	{ RBTREE_NULL, 0, (int (*)(const void *, const void *)) strcmp },
-	{ 0 }
+	{ NULL, {{ NULL, NULL, NULL }}}
 };
 getdns_dict *dnssec_ok_checking_disabled_roadblock_avoidance
     = &dnssec_ok_checking_disabled_roadblock_avoidance_spc;
 
 getdns_dict  dnssec_ok_checking_disabled_avoid_roadblocks_spc = {
 	{ RBTREE_NULL, 0, (int (*)(const void *, const void *)) strcmp },
-	{ 0 }
+	{ NULL, {{ NULL, NULL, NULL }}}
 };
 getdns_dict *dnssec_ok_checking_disabled_avoid_roadblocks
     = &dnssec_ok_checking_disabled_avoid_roadblocks_spc;
 
 
 static int
-is_extension_set(getdns_dict *extensions, const char *extension)
+is_extension_set(getdns_dict *extensions, const char *name, int default_value)
 {
 	getdns_return_t r;
 	uint32_t value;
 
-	if (! extensions)
-		return 0;
-	else if (extensions == dnssec_ok_checking_disabled
+	if ( ! extensions
+	    || extensions == dnssec_ok_checking_disabled
 	    || extensions == dnssec_ok_checking_disabled_roadblock_avoidance
 	    || extensions == dnssec_ok_checking_disabled_avoid_roadblocks)
 		return 0;
 
-	r = getdns_dict_get_int(extensions, extension, &value);
-	return r == GETDNS_RETURN_GOOD && value == GETDNS_EXTENSION_TRUE;
+	r = getdns_dict_get_int(extensions, name, &value);
+	return r == GETDNS_RETURN_GOOD ? ( value == GETDNS_EXTENSION_TRUE )
+	                               : default_value;
 }
 
 static void
@@ -108,6 +110,9 @@ network_req_cleanup(getdns_network_req *net_req)
 	if (net_req->response && (net_req->response < net_req->wire_data ||
 	    net_req->response > net_req->wire_data+ net_req->wire_data_sz))
 		GETDNS_FREE(net_req->owner->my_mf, net_req->response);
+	if (net_req->debug_tls_peer_cert.size &&
+	    net_req->debug_tls_peer_cert.data)
+		OPENSSL_free(net_req->debug_tls_peer_cert.data);
 }
 
 static uint8_t *
@@ -117,7 +122,7 @@ netreq_reset(getdns_network_req *net_req)
 	/* variables that need to be reset on reinit 
 	 */
 	net_req->unbound_id = -1;
-	net_req->state = NET_REQ_NOT_SENT;
+	_getdns_netreq_change_state(net_req, NET_REQ_NOT_SENT);
 	net_req->dnssec_status = GETDNS_DNSSEC_INDETERMINATE;
 	net_req->tsig_status = GETDNS_DNSSEC_INDETERMINATE;
 	net_req->query_id = 0;
@@ -143,7 +148,7 @@ network_req_init(getdns_network_req *net_req, getdns_dns_req *owner,
     int edns_maximum_udp_payload_size,
     uint8_t edns_extended_rcode, uint8_t edns_version, int edns_do_bit,
     uint16_t opt_options_size, size_t noptions, getdns_list *options,
-    size_t wire_data_sz, size_t max_query_sz)
+    size_t wire_data_sz, size_t max_query_sz, getdns_dict *extensions)
 {
 	uint8_t *buf;
 	getdns_dict    *option;
@@ -151,6 +156,7 @@ network_req_init(getdns_network_req *net_req, getdns_dns_req *owner,
 	getdns_bindata *option_data;
 	size_t i;
 	int r = 0;
+	gldns_buffer gbuf;
 
 	/* variables that stay the same on reinit, don't touch
 	 */
@@ -167,19 +173,26 @@ network_req_init(getdns_network_req *net_req, getdns_dns_req *owner,
 	    net_req->transport_count * sizeof(getdns_transport_list_t));
 	net_req->tls_auth_min = owner->context->tls_auth_min;
 
+	net_req->follow_redirects = owner->context->follow_redirects;
+
 	/* state variables from the resolver, don't touch
 	 */
 	net_req->upstream = NULL;
 	net_req->fd = -1;
 	net_req->transport_current = 0;
 	memset(&net_req->event, 0, sizeof(net_req->event));
-	memset(&net_req->tcp, 0, sizeof(net_req->tcp));
 	net_req->keepalive_sent = 0;
 	net_req->write_queue_tail = NULL;
 	/* Some fields to record info for return_call_reporting */
-	net_req->debug_tls_auth_status = 0;
+	net_req->debug_tls_auth_status = GETDNS_AUTH_NONE;
+	net_req->debug_tls_peer_cert.size = 0;
+	net_req->debug_tls_peer_cert.data = NULL;
 	net_req->debug_udp = 0;
 
+	/* Scheduling, touch only via _getdns_netreq_change_state!
+	 */
+	net_req->state = NET_REQ_NOT_SENT;
+
 	if (max_query_sz == 0) {
 		net_req->query    = NULL;
 		net_req->opt      = NULL;
@@ -193,8 +206,6 @@ network_req_init(getdns_network_req *net_req, getdns_dns_req *owner,
 	buf = net_req->query;
 	gldns_write_uint16(buf + 2, 0); /* reset all flags */
 	GLDNS_RD_SET(buf);
-	if (dnssec_extension_set) /* We will do validation ourselves */
-		GLDNS_CD_SET(buf);
 	GLDNS_OPCODE_SET(buf, GLDNS_PACKET_QUERY);
 	gldns_write_uint16(buf + GLDNS_QDCOUNT_OFF, 1); /* 1 query */
 	gldns_write_uint16(buf + GLDNS_ANCOUNT_OFF, 0); /* 0 answers */
@@ -202,6 +213,15 @@ network_req_init(getdns_network_req *net_req, getdns_dns_req *owner,
 	gldns_write_uint16(buf + GLDNS_ARCOUNT_OFF, with_opt ? 1 : 0);
 
 	buf = netreq_reset(net_req);
+	gldns_buffer_init_frm_data(
+	    &gbuf, net_req->query, net_req->wire_data_sz - 2);
+	if (owner->context->header)
+		_getdns_reply_dict2wire(owner->context->header, &gbuf, 1);
+	gldns_buffer_rewind(&gbuf);
+	_getdns_reply_dict2wire(extensions, &gbuf, 1);
+	if (dnssec_extension_set) /* We will do validation ourselves */
+		GLDNS_CD_SET(net_req->query);
+
 	if (with_opt) {
 		net_req->opt = buf;
 		buf[0] = 0; /* dname for . */
@@ -252,10 +272,10 @@ _getdns_network_req_clear_upstream_options(getdns_network_req * req)
 {
   size_t pktlen;
   if (req->opt) {
-	  gldns_write_uint16(req->opt + 9, req->base_query_option_sz);
+	  gldns_write_uint16(req->opt + 9, (uint16_t) req->base_query_option_sz);
 	  req->response = req->opt + 11 + req->base_query_option_sz;
 	  pktlen = req->response - req->query;
-	  gldns_write_uint16(req->query - 2, pktlen);
+	  gldns_write_uint16(req->query - 2, (uint16_t) pktlen);
   }
 }
 
@@ -373,7 +393,7 @@ _getdns_network_req_add_tsig(getdns_network_req *req)
 #endif
 	tsig_info = _getdns_get_tsig_info(upstream->tsig_alg);
 
-	gldns_buffer_init_frm_data(&gbuf, req->response, MAXIMUM_TSIG_SPACE);
+	gldns_buffer_init_vfixed_frm_data(&gbuf, req->response, MAXIMUM_TSIG_SPACE);
 	gldns_buffer_write(&gbuf,
 	    upstream->tsig_dname, upstream->tsig_dname_len);	/* Name */
 	gldns_buffer_write_u16(&gbuf, GETDNS_RRCLASS_ANY);	/* Class */
@@ -418,7 +438,7 @@ _getdns_network_req_add_tsig(getdns_network_req *req)
 	gldns_buffer_write_u16(&gbuf, GETDNS_RRCLASS_ANY);	/* Class */
 	gldns_buffer_write_u32(&gbuf, 0);			/* TTL */
 	gldns_buffer_write_u16(&gbuf,
-	    tsig_info->dname_len + 10 + md_len + 6);	/* RdLen */
+	    (uint16_t)(tsig_info->dname_len + 10 + md_len + 6));	/* RdLen */
 	gldns_buffer_write(&gbuf,
 	    tsig_info->dname, tsig_info->dname_len);	/* Algorithm Name */
 	gldns_buffer_write_u48(&gbuf, time(NULL));	/* Time Signed */
@@ -459,15 +479,18 @@ _getdns_network_validate_tsig(getdns_network_req *req)
 	unsigned int result_mac_len = EVP_MAX_MD_SIZE;
 	uint16_t original_id;
 	const EVP_MD *digester;
-	HMAC_CTX ctx;
+	HMAC_CTX *ctx;
+#ifndef HAVE_HMAC_CTX_NEW
+	HMAC_CTX ctx_space;
+#endif
 
-	DEBUG_STUB("Validate TSIG\n");
+	DEBUG_STUB("%s %-35s: Validate TSIG\n", STUB_DEBUG_TSIG, __FUNC__);
 	for ( rr = _getdns_rr_iter_init(&rr_spc, req->query,
 	                                (req->response - req->query))
 	    ; rr
 	    ; rr = _getdns_rr_iter_next(rr)) {
 
-		if (_getdns_rr_iter_section(rr) == GLDNS_SECTION_ADDITIONAL &&
+		if (_getdns_rr_iter_section(rr) == SECTION_ADDITIONAL &&
 		    gldns_read_uint16(rr->rr_type) == GETDNS_RRTYPE_TSIG)
 			break;
 	}
@@ -477,7 +500,9 @@ _getdns_network_validate_tsig(getdns_network_req *req)
 	request_mac_len = gldns_read_uint16(rdf->pos);
 	if (request_mac_len != rdf->nxt - rdf->pos - 2)
 		return;
-	DEBUG_STUB("Request MAC found length: %d\n", (int)(request_mac_len));
+	DEBUG_STUB("%s %-35s: Request MAC found length %d\n",
+	           STUB_DEBUG_TSIG, __FUNC__, (int)(request_mac_len));
+	
 	request_mac = rdf->pos + 2;
 
 	/* Now we expect a TSIG on the response! */
@@ -488,7 +513,7 @@ _getdns_network_validate_tsig(getdns_network_req *req)
 	    ; rr
 	    ; rr = _getdns_rr_iter_next(rr)) {
 
-		if (_getdns_rr_iter_section(rr) == GLDNS_SECTION_ADDITIONAL &&
+		if (_getdns_rr_iter_section(rr) == SECTION_ADDITIONAL &&
 		    gldns_read_uint16(rr->rr_type) == GETDNS_RRTYPE_TSIG)
 			break;
 	}
@@ -532,7 +557,8 @@ _getdns_network_validate_tsig(getdns_network_req *req)
 	response_mac_len = gldns_read_uint16(rdf->pos);
 	if (response_mac_len != rdf->nxt - rdf->pos - 2)
 		return;
-	DEBUG_STUB("Response MAC found length: %d\n", (int)(response_mac_len));
+	DEBUG_STUB("%s %-35s: Response MAC found length: %d\n",
+	           STUB_DEBUG_TSIG, __FUNC__, (int)(response_mac_len));
 	response_mac = rdf->pos + 2;
 
 	if (!(rdf = _getdns_rdf_iter_next(rdf)) ||
@@ -549,14 +575,15 @@ _getdns_network_validate_tsig(getdns_network_req *req)
 		return;
 
 	gldns_buffer_write_u16(&gbuf, 0);		/* Other len */
-	other_len = gldns_read_uint16(rdf->pos);
+	other_len = (uint8_t) gldns_read_uint16(rdf->pos);
 	if (other_len != rdf->nxt - rdf->pos - 2)
 		return;
 	if (other_len)
 		gldns_buffer_write(&gbuf, rdf->pos, other_len);
 
 	/* TSIG found */
-	DEBUG_STUB("TSIG found, original ID: %d\n", (int)original_id);
+	DEBUG_STUB("%s %-35s: TSIG found, original ID: %d\n",
+	           STUB_DEBUG_TSIG, __FUNC__, (int)original_id);
 
 	gldns_write_uint16(req->response + 10,
 	    gldns_read_uint16(req->response + 10) - 1);
@@ -583,22 +610,30 @@ _getdns_network_validate_tsig(getdns_network_req *req)
 #endif
 	default                : return;
 	}
-	
-	HMAC_CTX_init(&ctx);
-	(void) HMAC_Init_ex(&ctx, req->upstream->tsig_key,
+#ifdef HAVE_HMAC_CTX_NEW
+	ctx = HMAC_CTX_new();
+#else
+	ctx = &ctx_space;
+	HMAC_CTX_init(ctx);
+#endif	
+	(void) HMAC_Init_ex(ctx, req->upstream->tsig_key,
 	    req->upstream->tsig_size, digester, NULL);
-	(void) HMAC_Update(&ctx, request_mac - 2, request_mac_len + 2);
-	(void) HMAC_Update(&ctx, req->response, rr->pos - req->response);
-	(void) HMAC_Update(&ctx, tsig_vars, gldns_buffer_position(&gbuf));
-	HMAC_Final(&ctx, result_mac, &result_mac_len);
+	(void) HMAC_Update(ctx, request_mac - 2, request_mac_len + 2);
+	(void) HMAC_Update(ctx, req->response, rr->pos - req->response);
+	(void) HMAC_Update(ctx, tsig_vars, gldns_buffer_position(&gbuf));
+	HMAC_Final(ctx, result_mac, &result_mac_len);
 
-	DEBUG_STUB("Result MAC length: %d\n", (int)(result_mac_len));
+	DEBUG_STUB("%s %-35s: Result MAC length: %d\n",
+	           STUB_DEBUG_TSIG, __FUNC__, (int)(result_mac_len));
 	if (result_mac_len == response_mac_len &&
 	    memcmp(result_mac, response_mac, result_mac_len) == 0)
 		req->tsig_status = GETDNS_DNSSEC_SECURE;
 
-	HMAC_CTX_cleanup(&ctx);
-
+#ifdef HAVE_HMAC_CTX_FREE
+	HMAC_CTX_free(ctx);
+#else
+	HMAC_CTX_cleanup(ctx);
+#endif
 	gldns_write_uint16(req->response, gldns_read_uint16(req->query));
 	gldns_write_uint16(req->response + 10,
 	    gldns_read_uint16(req->response + 10) + 1);
@@ -619,11 +654,12 @@ _getdns_dns_req_free(getdns_dns_req * req)
 		network_req_cleanup(*net_req);
 
 	/* clear timeout event */
-	if (req->timeout.timeout_cb) {
+	if (req->loop && req->loop->vmt && req->timeout.timeout_cb) {
 		req->loop->vmt->clear(req->loop, &req->timeout);
 		req->timeout.timeout_cb = NULL;
 	}
-
+	if (req->freed)
+		*req->freed = 1;
 	GETDNS_FREE(req->my_mf, req);
 }
 
@@ -632,28 +668,39 @@ static const uint8_t no_suffixes[] = { 1, 0 };
 /* create a new dns req to be submitted */
 getdns_dns_req *
 _getdns_dns_req_new(getdns_context *context, getdns_eventloop *loop,
-    const char *name, uint16_t request_type, getdns_dict *extensions)
+    const char *name, uint16_t request_type, getdns_dict *extensions,
+    uint64_t *now_ms)
 {
-	int dnssec_return_status
-	    =  context->return_dnssec_status == GETDNS_EXTENSION_TRUE
-	    || is_extension_set(extensions, "dnssec_return_status");
-	int dnssec_return_only_secure
-	    =  is_extension_set(extensions, "dnssec_return_only_secure");
-	int dnssec_return_validation_chain
-	    =  is_extension_set(extensions, "dnssec_return_validation_chain");
-	int edns_cookies
-	    =  is_extension_set(extensions, "edns_cookies");
+	int dnssec_return_status                 = is_extension_set(
+	    extensions, "dnssec_return_status",
+	        context->dnssec_return_status);
+	int dnssec_return_only_secure            = is_extension_set(
+	    extensions, "dnssec_return_only_secure",
+	        context->dnssec_return_only_secure);
+	int dnssec_return_all_statuses           = is_extension_set(
+	    extensions, "dnssec_return_all_statuses",
+	        context->dnssec_return_all_statuses);
+	int  dnssec_return_full_validation_chain = is_extension_set(
+	    extensions, "dnssec_return_full_validation_chain",
+	        context->dnssec_return_full_validation_chain);
+	int dnssec_return_validation_chain       = is_extension_set(
+	    extensions, "dnssec_return_validation_chain",
+	        context->dnssec_return_validation_chain);
+	int edns_cookies                         = is_extension_set(
+	    extensions, "edns_cookies",
+	        context->edns_cookies);
 #ifdef DNSSEC_ROADBLOCK_AVOIDANCE
 	int avoid_dnssec_roadblocks
 	    =  (extensions == dnssec_ok_checking_disabled_avoid_roadblocks);
-	int dnssec_roadblock_avoidance
-	    = is_extension_set(extensions, "dnssec_roadblock_avoidance")
+	int dnssec_roadblock_avoidance = avoid_dnssec_roadblocks
 	    || (extensions == dnssec_ok_checking_disabled_roadblock_avoidance)
-	    || avoid_dnssec_roadblocks;
+	    || is_extension_set(extensions, "dnssec_roadblock_avoidance",
+	                            context->dnssec_roadblock_avoidance);
 #endif
-
 	int dnssec_extension_set = dnssec_return_status
-	    || dnssec_return_only_secure || dnssec_return_validation_chain
+	    || dnssec_return_only_secure || dnssec_return_all_statuses
+	    || dnssec_return_validation_chain
+	    || dnssec_return_full_validation_chain
 	    || (extensions == dnssec_ok_checking_disabled)
 	    || (extensions == dnssec_ok_checking_disabled_roadblock_avoidance)
 	    || (extensions == dnssec_ok_checking_disabled_avoid_roadblocks)
@@ -671,7 +718,7 @@ _getdns_dns_req_new(getdns_context *context, getdns_eventloop *loop,
 	getdns_dict *add_opt_parameters;
 	int     have_add_opt_parameters;
 
-	getdns_list *options;
+	getdns_list *options = NULL;
 	size_t      noptions = 0;
 	size_t       i;
 
@@ -683,9 +730,9 @@ _getdns_dns_req_new(getdns_context *context, getdns_eventloop *loop,
 	int with_opt;
 
 	getdns_dns_req *result = NULL;
-        uint32_t klass = GLDNS_RR_CLASS_IN;
-	int a_aaaa_query =
-	    is_extension_set(extensions, "return_both_v4_and_v6") &&
+        uint32_t klass = context->specify_class;
+	int a_aaaa_query = is_extension_set(extensions,
+	    "return_both_v4_and_v6", context->return_both_v4_and_v6) &&
 	    ( request_type == GETDNS_RRTYPE_A ||
 	      request_type == GETDNS_RRTYPE_AAAA );
 	/* Reserve for the buffer at least one more byte
@@ -702,7 +749,10 @@ _getdns_dns_req_new(getdns_context *context, getdns_eventloop *loop,
 
 	have_add_opt_parameters = getdns_dict_get_dict(extensions,
 	    "add_opt_parameters", &add_opt_parameters) == GETDNS_RETURN_GOOD;
-
+	if (!have_add_opt_parameters && context->add_opt_parameters) {
+		add_opt_parameters = context->add_opt_parameters;
+		have_add_opt_parameters = 1;
+	}
 	if (dnssec_extension_set) {
 		edns_maximum_udp_payload_size = -1;
 		edns_extended_rcode = 0;
@@ -716,17 +766,26 @@ _getdns_dns_req_new(getdns_context *context, getdns_eventloop *loop,
 		edns_do_bit = context->edns_do_bit;
 
 		if (have_add_opt_parameters) {
-			if (!getdns_dict_get_int(add_opt_parameters,
+			if (getdns_dict_get_int(add_opt_parameters,
 			    "maximum_udp_payload_size",
-			    &get_edns_maximum_udp_payload_size))
+			    &get_edns_maximum_udp_payload_size)) {
+				if (!getdns_dict_get_int(
+				    add_opt_parameters, "udp_payload_size",
+				    &get_edns_maximum_udp_payload_size))
+					edns_maximum_udp_payload_size =
+					    get_edns_maximum_udp_payload_size;
+			} else
 				edns_maximum_udp_payload_size =
 				    get_edns_maximum_udp_payload_size;
+
 			(void) getdns_dict_get_int(add_opt_parameters,
 			    "extended_rcode", &edns_extended_rcode);
 			(void) getdns_dict_get_int(add_opt_parameters,
 			    "version", &edns_version);
-			(void) getdns_dict_get_int(add_opt_parameters,
-			    "do_bit", &edns_do_bit);
+			if (getdns_dict_get_int(add_opt_parameters,
+			    "do_bit", &edns_do_bit))
+				(void) getdns_dict_get_int(
+				    add_opt_parameters, "do", &edns_do_bit);
 		}
 	}
 	if (have_add_opt_parameters && getdns_dict_get_list(
@@ -820,7 +879,9 @@ _getdns_dns_req_new(getdns_context *context, getdns_eventloop *loop,
 		GETDNS_FREE(result->my_mf, result);
 		return NULL;
 	}
-	if (result->append_name == GETDNS_APPEND_NAME_ALWAYS) {
+	if (result->append_name == GETDNS_APPEND_NAME_ALWAYS ||
+	    (  result->append_name == GETDNS_APPEND_NAME_TO_SINGLE_LABEL_FIRST
+	    && result->name[0] && result->name[result->name[0]+1] == 0)){
 		for (
 		    ; result->suffix_len > 1 && *result->suffix
 		    ; result->suffix += result->suffix_len
@@ -837,6 +898,7 @@ _getdns_dns_req_new(getdns_context *context, getdns_eventloop *loop,
 		}
 	} else if (result->append_name ==
 	    GETDNS_APPEND_NAME_ONLY_TO_SINGLE_LABEL_AFTER_FAILURE &&
+	    result->name[0] &&
 	    result->name[result->name[0]+1] != 0) {
 		/* We have multiple labels, no appending */
 		result->suffix_len = no_suffixes[0];
@@ -845,12 +907,14 @@ _getdns_dns_req_new(getdns_context *context, getdns_eventloop *loop,
 	}
 	result->context = context;
 	result->loop = loop;
-	result->canceled = 0;
-	result->trans_id = (((uint64_t)arc4random()) << 32) |
-	                    ((uint64_t)arc4random());
+	result->trans_id = (uint64_t) (intptr_t) result;
 	result->dnssec_return_status           = dnssec_return_status;
 	result->dnssec_return_only_secure      = dnssec_return_only_secure;
-	result->dnssec_return_validation_chain = dnssec_return_validation_chain;
+	result->dnssec_return_all_statuses     = dnssec_return_all_statuses;
+	result->dnssec_return_full_validation_chain =
+		dnssec_return_full_validation_chain;
+	result->dnssec_return_validation_chain = dnssec_return_validation_chain
+	     || dnssec_return_full_validation_chain;
 	result->edns_cookies                   = edns_cookies;
 #ifdef DNSSEC_ROADBLOCK_AVOIDANCE
 	result->dnssec_roadblock_avoidance     = dnssec_roadblock_avoidance;
@@ -858,10 +922,10 @@ _getdns_dns_req_new(getdns_context *context, getdns_eventloop *loop,
 #endif
 	result->edns_client_subnet_private     = context->edns_client_subnet_private;
 	result->tls_query_padding_blocksize    = context->tls_query_padding_blocksize;
-	result->return_call_reporting          =
-	    is_extension_set(extensions, "return_call_reporting");
-	result->add_warning_for_bad_dns        =
-	    is_extension_set(extensions, "add_warning_for_bad_dns");
+	result->return_call_reporting          = is_extension_set(extensions,
+	    "return_call_reporting"  , context->return_call_reporting);
+	result->add_warning_for_bad_dns        = is_extension_set(extensions,
+	    "add_warning_for_bad_dns", context->add_warning_for_bad_dns);
 	
 	/* will be set by caller */
 	result->user_pointer = NULL;
@@ -876,12 +940,20 @@ _getdns_dns_req_new(getdns_context *context, getdns_eventloop *loop,
 	if (result->upstreams)
 		result->upstreams->referenced++;
 
+	result->finished_next = NULL;
+	result->freed = NULL;
+	result->validating = 0;
+	result->is_dns_request = 1;
+	result->request_timed_out = 0;
+	result->chain = NULL;
+
 	network_req_init(result->netreqs[0], result,
 	    request_type, dnssec_extension_set, with_opt,
 	    edns_maximum_udp_payload_size,
 	    edns_extended_rcode, edns_version, edns_do_bit,
-	    opt_options_size, noptions, options,
-	    netreq_sz - sizeof(getdns_network_req), max_query_sz);
+	    (uint16_t) opt_options_size, noptions, options,
+	    netreq_sz - sizeof(getdns_network_req), max_query_sz,
+	    extensions);
 
 	if (a_aaaa_query)
 		network_req_init(result->netreqs[1], result,
@@ -890,8 +962,14 @@ _getdns_dns_req_new(getdns_context *context, getdns_eventloop *loop,
 		    dnssec_extension_set, with_opt,
 		    edns_maximum_udp_payload_size,
 		    edns_extended_rcode, edns_version, edns_do_bit,
-		    opt_options_size, noptions, options,
-		    netreq_sz - sizeof(getdns_network_req), max_query_sz);
+		    (uint16_t) opt_options_size, noptions, options,
+		    netreq_sz - sizeof(getdns_network_req), max_query_sz,
+		    extensions);
+
+	if (*now_ms == 0 && (*now_ms = _getdns_get_now_ms()) == 0)
+		result->expires = 0;
+	else
+		result->expires = *now_ms + context->timeout;
 
 	return result;
 }
