@@ -29,7 +29,14 @@
 #include <stdlib.h>
 #include <string.h>
 #include <inttypes.h>
+#ifdef __clang__
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wgnu-zero-variadic-macro-arguments"
+#endif
 #include <check.h>
+#ifdef __clang__
+#pragma clang diagnostic pop
+#endif
 #include "getdns/getdns.h"
 #include "config.h"
 #include "check_getdns_common.h"
@@ -320,6 +327,7 @@ void destroy_callbackfn(struct getdns_context *context,
                         void *userarg,
                         getdns_transaction_t transaction_id) {
     int* flag = (int*)userarg;
+    (void)callback_type; (void)transaction_id;
     *flag = 1;
     getdns_dict_destroy(response);
     getdns_context_destroy(context);
@@ -338,7 +346,8 @@ void callbackfn(struct getdns_context *context,
                 getdns_transaction_t transaction_id)
 {
   typedef void (*fn_ptr)(struct extracted_response *ex_response);
-  fn_ptr fn = userarg;
+  fn_ptr fn = ((fn_cont *)userarg)->fn;
+  (void)context; (void)transaction_id;
 
   /*
    *  If userarg is NULL, either a negative test case
@@ -378,7 +387,7 @@ void callbackfn(struct getdns_context *context,
 void update_callbackfn(struct getdns_context *context,
                 getdns_context_code_t changed_item)
 {
-
+  (void)context;
   ck_assert_msg(changed_item == expected_changed_item,
     "Expected changed_item == %d, got %d",
     changed_item, expected_changed_item);
