@@ -41,6 +41,7 @@
 #include "config.h"
 #include "context.h"
 #include "rr-iter.h"
+#include <ctype.h>
 
 #define UNCONST_UINT8_p uint8_t *
 
@@ -56,7 +57,8 @@
 
 struct ub_result;
 struct getdns_network_req;
-getdns_return_t getdns_apply_network_result(getdns_network_req* netreq, struct ub_result* result);
+getdns_return_t getdns_apply_network_result(getdns_network_req* netreq,
+    int rcode, void *pkt, int pkt_len, int sec, char* why_bogus);
 
 #define GETDNS_MAX_DNAME_LEN 255
 #define GETDNS_MAX_LABEL_LEN 63
@@ -66,16 +68,16 @@ getdns_return_t getdns_apply_network_result(getdns_network_req* netreq, struct u
  * description but the list_set functions seem to be designed to modify an existing
  * item in the list.  The newly added item has no data type.
  * @param list list containing the item to which child_list is to be added
- * @param *index assigned to the index of the newly added item on success
+ * @param *child_dict dict to append to the list
  * @return GETDNS_RETURN_GOOD on success
  * @return GETDNS_RETURN_GENERAL_ERROR if out of memory
  */
+getdns_return_t _getdns_list_append_this_dict(getdns_list *list,
+    getdns_dict *child_dict);
+
 getdns_return_t _getdns_list_append_dict(getdns_list *list,
     const getdns_dict *child_dict);
-getdns_return_t _getdns_list_append_list(getdns_list *list,
-    const getdns_list *child_list);
-getdns_return_t _getdns_list_append_bindata(getdns_list *list,
-    const getdns_bindata *child_bindata);
+
 getdns_return_t _getdns_list_append_string(getdns_list *list,
     const char *value);
 
@@ -85,9 +87,18 @@ getdns_return_t _getdns_list_append_int(getdns_list *list,
 getdns_return_t _getdns_list_append_const_bindata(getdns_list *list,
     size_t size, const void *data);
 
+getdns_return_t _getdns_dict_set_this_dict(getdns_dict *dict,
+    const char *name, getdns_dict *child_dict);
+
+getdns_return_t _getdns_dict_set_this_list(getdns_dict *dict,
+    const char *name, getdns_list *child_list);
+
 getdns_return_t _getdns_dict_set_const_bindata(getdns_dict *dict,
     const char *name, size_t size, const void *data);
 
+getdns_return_t _getdns_dict_set_this_bindata(getdns_dict *dict,
+    const char *name, getdns_bindata *child_bindata);
+
 /**
   * private function (API users should not be calling this), this uses library
   * routines to make a copy of the list - would be faster to make the copy directly
@@ -131,16 +142,15 @@ getdns_return_t _getdns_sockaddr_to_dict(struct getdns_context *context,
 getdns_dict *
 _getdns_rr_iter2rr_dict(struct mem_funcs *mf, _getdns_rr_iter *i);
 
+getdns_dict *
+_getdns_rr_iter2rr_dict_canonical(
+    struct mem_funcs *mf, _getdns_rr_iter *i, uint32_t *orig_ttl);
+
 struct getdns_dns_req;
 struct getdns_dict *_getdns_create_getdns_response(struct getdns_dns_req *completed_request);
 
-getdns_dict *_getdns_create_reply_dict(getdns_context *context,
-    getdns_network_req *req, getdns_list *just_addrs, int *rrsigs_in_answer);
-
 getdns_return_t _getdns_validate_dname(const char* dname);
 
-int _getdns_dname_equal(const uint8_t *s1, const uint8_t *s2);
-
 uint8_t *_getdns_list2wire(
     getdns_list *l, uint8_t *buf, size_t *buf_len, struct mem_funcs *mf);
 
@@ -171,5 +181,42 @@ INLINE getdns_eventloop_event *getdns_eventloop_event_init(
 #define GETDNS_SCHEDULE_EVENT(loop, fd, timeout, event) \
 	do { (loop)->vmt->schedule((loop),(fd),(timeout),(event)); } while(0)
 
+INLINE void _dname_canonicalize(const uint8_t *src, uint8_t *dst)
+{
+	const uint8_t *next_label;
+
+	while (*src) {
+		next_label = src + *src + 1;
+		*dst++ = *src++;
+		while (src < next_label)
+			*dst++ = (uint8_t)tolower((unsigned char)*src++);
+	}
+}
+
+INLINE void _dname_canonicalize2(uint8_t *dname)
+{
+	_dname_canonicalize(dname, dname);
+}
+
+INLINE uint64_t _getdns_get_now_ms()
+{
+	struct timeval tv;
+
+	(void) gettimeofday(&tv, NULL);
+	return tv.tv_sec * 1000 + tv.tv_usec / 1000;
+}
+
+INLINE uint64_t _getdns_ms_until_expiry(uint64_t expires)
+{
+	uint64_t now_ms = _getdns_get_now_ms();
+	return now_ms >= expires ? 0 : expires - now_ms;
+}
+
+INLINE uint64_t _getdns_ms_until_expiry2(uint64_t expires, uint64_t *now_ms)
+{
+	if (*now_ms == 0) *now_ms = _getdns_get_now_ms();
+	return *now_ms >= expires ? 0 : expires - *now_ms;
+}
+
 #endif
 /* util-internal.h */
